@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,20 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core.client_options import ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.cloud.debugger_v2.types import controller
 from google.cloud.debugger_v2.types import data
@@ -119,6 +122,42 @@ class Controller2AsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return Controller2Client.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> Controller2Transport:
         """Returns the transport used by the client instance.
@@ -201,6 +240,25 @@ class Controller2AsyncClient:
         format. Agents must handle ``debuggee_id`` value changing upon
         re-registration.
 
+
+        .. code-block:: python
+
+            from google.cloud import debugger_v2
+
+            def sample_register_debuggee():
+                # Create a client
+                client = debugger_v2.Controller2Client()
+
+                # Initialize request argument(s)
+                request = debugger_v2.RegisterDebuggeeRequest(
+                )
+
+                # Make the request
+                response = client.register_debuggee(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.debugger_v2.types.RegisterDebuggeeRequest, dict]):
                 The request object. Request to register a debuggee.
@@ -223,7 +281,7 @@ class Controller2AsyncClient:
                 Response for registering a debuggee.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([debuggee])
         if request is not None and has_flattened_params:
@@ -277,6 +335,26 @@ class Controller2AsyncClient:
         breakpoints that are completed until the controller removes them
         from the active list to avoid setting those breakpoints again.
 
+
+        .. code-block:: python
+
+            from google.cloud import debugger_v2
+
+            def sample_list_active_breakpoints():
+                # Create a client
+                client = debugger_v2.Controller2Client()
+
+                # Initialize request argument(s)
+                request = debugger_v2.ListActiveBreakpointsRequest(
+                    debuggee_id="debuggee_id_value",
+                )
+
+                # Make the request
+                response = client.list_active_breakpoints(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.debugger_v2.types.ListActiveBreakpointsRequest, dict]):
                 The request object. Request to list active breakpoints.
@@ -298,7 +376,7 @@ class Controller2AsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([debuggee_id])
         if request is not None and has_flattened_params:
@@ -358,6 +436,26 @@ class Controller2AsyncClient:
         changes such as canonicalizing a value or snapping the location
         to the correct line of code.
 
+
+        .. code-block:: python
+
+            from google.cloud import debugger_v2
+
+            def sample_update_active_breakpoint():
+                # Create a client
+                client = debugger_v2.Controller2Client()
+
+                # Initialize request argument(s)
+                request = debugger_v2.UpdateActiveBreakpointRequest(
+                    debuggee_id="debuggee_id_value",
+                )
+
+                # Make the request
+                response = client.update_active_breakpoint(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.debugger_v2.types.UpdateActiveBreakpointRequest, dict]):
                 The request object. Request to update an active
@@ -391,7 +489,7 @@ class Controller2AsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([debuggee_id, breakpoint_])
         if request is not None and has_flattened_params:
