@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -240,20 +240,20 @@ def test_controller2_client_client_options(
     # unsupported value.
     with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
         with pytest.raises(MutualTLSChannelError):
-            client = client_class()
+            client = client_class(transport=transport_name)
 
     # Check the case GOOGLE_API_USE_CLIENT_CERTIFICATE has unsupported value.
     with mock.patch.dict(
         os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
     ):
         with pytest.raises(ValueError):
-            client = client_class()
+            client = client_class(transport=transport_name)
 
     # Check the case quota_project_id is provided
     options = client_options.ClientOptions(quota_project_id="octopus")
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(transport=transport_name, client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -310,7 +310,7 @@ def test_controller2_client_mtls_env_auto(
         )
         with mock.patch.object(transport_class, "__init__") as patched:
             patched.return_value = None
-            client = client_class(transport=transport_name, client_options=options)
+            client = client_class(client_options=options, transport=transport_name)
 
             if use_client_cert_env == "false":
                 expected_client_cert_source = None
@@ -387,6 +387,83 @@ def test_controller2_client_mtls_env_auto(
                 )
 
 
+@pytest.mark.parametrize("client_class", [Controller2Client, Controller2AsyncClient])
+@mock.patch.object(
+    Controller2Client, "DEFAULT_ENDPOINT", modify_default_endpoint(Controller2Client)
+)
+@mock.patch.object(
+    Controller2AsyncClient,
+    "DEFAULT_ENDPOINT",
+    modify_default_endpoint(Controller2AsyncClient),
+)
+def test_controller2_client_get_mtls_endpoint_and_cert_source(client_class):
+    mock_client_cert_source = mock.Mock()
+
+    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "true".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        mock_api_endpoint = "foo"
+        options = client_options.ClientOptions(
+            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
+        )
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
+            options
+        )
+        assert api_endpoint == mock_api_endpoint
+        assert cert_source == mock_client_cert_source
+
+    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "false".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "false"}):
+        mock_client_cert_source = mock.Mock()
+        mock_api_endpoint = "foo"
+        options = client_options.ClientOptions(
+            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
+        )
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
+            options
+        )
+        assert api_endpoint == mock_api_endpoint
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "never".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+        assert api_endpoint == client_class.DEFAULT_ENDPOINT
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "always".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+        assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert doesn't exist.
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        with mock.patch(
+            "google.auth.transport.mtls.has_default_client_cert_source",
+            return_value=False,
+        ):
+            api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+            assert api_endpoint == client_class.DEFAULT_ENDPOINT
+            assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert exists.
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        with mock.patch(
+            "google.auth.transport.mtls.has_default_client_cert_source",
+            return_value=True,
+        ):
+            with mock.patch(
+                "google.auth.transport.mtls.default_client_cert_source",
+                return_value=mock_client_cert_source,
+            ):
+                (
+                    api_endpoint,
+                    cert_source,
+                ) = client_class.get_mtls_endpoint_and_cert_source()
+                assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
+                assert cert_source == mock_client_cert_source
+
+
 @pytest.mark.parametrize(
     "client_class,transport_class,transport_name",
     [
@@ -405,7 +482,7 @@ def test_controller2_client_client_options_scopes(
     options = client_options.ClientOptions(scopes=["1", "2"],)
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(transport=transport_name, client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -419,24 +496,26 @@ def test_controller2_client_client_options_scopes(
 
 
 @pytest.mark.parametrize(
-    "client_class,transport_class,transport_name",
+    "client_class,transport_class,transport_name,grpc_helpers",
     [
-        (Controller2Client, transports.Controller2GrpcTransport, "grpc"),
+        (Controller2Client, transports.Controller2GrpcTransport, "grpc", grpc_helpers),
         (
             Controller2AsyncClient,
             transports.Controller2GrpcAsyncIOTransport,
             "grpc_asyncio",
+            grpc_helpers_async,
         ),
     ],
 )
 def test_controller2_client_client_options_credentials_file(
-    client_class, transport_class, transport_name
+    client_class, transport_class, transport_name, grpc_helpers
 ):
     # Check the case credentials file is provided.
     options = client_options.ClientOptions(credentials_file="credentials.json")
+
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(transport=transport_name, client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file="credentials.json",
@@ -467,9 +546,72 @@ def test_controller2_client_client_options_from_dict():
         )
 
 
-def test_register_debuggee(
-    transport: str = "grpc", request_type=controller.RegisterDebuggeeRequest
+@pytest.mark.parametrize(
+    "client_class,transport_class,transport_name,grpc_helpers",
+    [
+        (Controller2Client, transports.Controller2GrpcTransport, "grpc", grpc_helpers),
+        (
+            Controller2AsyncClient,
+            transports.Controller2GrpcAsyncIOTransport,
+            "grpc_asyncio",
+            grpc_helpers_async,
+        ),
+    ],
+)
+def test_controller2_client_create_channel_credentials_file(
+    client_class, transport_class, transport_name, grpc_helpers
 ):
+    # Check the case credentials file is provided.
+    options = client_options.ClientOptions(credentials_file="credentials.json")
+
+    with mock.patch.object(transport_class, "__init__") as patched:
+        patched.return_value = None
+        client = client_class(client_options=options, transport=transport_name)
+        patched.assert_called_once_with(
+            credentials=None,
+            credentials_file="credentials.json",
+            host=client.DEFAULT_ENDPOINT,
+            scopes=None,
+            client_cert_source_for_mtls=None,
+            quota_project_id=None,
+            client_info=transports.base.DEFAULT_CLIENT_INFO,
+            always_use_jwt_access=True,
+        )
+
+    # test that the credentials from file are saved and used as the credentials.
+    with mock.patch.object(
+        google.auth, "load_credentials_from_file", autospec=True
+    ) as load_creds, mock.patch.object(
+        google.auth, "default", autospec=True
+    ) as adc, mock.patch.object(
+        grpc_helpers, "create_channel"
+    ) as create_channel:
+        creds = ga_credentials.AnonymousCredentials()
+        file_creds = ga_credentials.AnonymousCredentials()
+        load_creds.return_value = (file_creds, None)
+        adc.return_value = (creds, None)
+        client = client_class(client_options=options, transport=transport_name)
+        create_channel.assert_called_with(
+            "clouddebugger.googleapis.com:443",
+            credentials=file_creds,
+            credentials_file=None,
+            quota_project_id=None,
+            default_scopes=(
+                "https://www.googleapis.com/auth/cloud-platform",
+                "https://www.googleapis.com/auth/cloud_debugger",
+            ),
+            scopes=None,
+            default_host="clouddebugger.googleapis.com",
+            ssl_credentials=None,
+            options=[
+                ("grpc.max_send_message_length", -1),
+                ("grpc.max_receive_message_length", -1),
+            ],
+        )
+
+
+@pytest.mark.parametrize("request_type", [controller.RegisterDebuggeeRequest, dict,])
+def test_register_debuggee(request_type, transport: str = "grpc"):
     client = Controller2Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -495,10 +637,6 @@ def test_register_debuggee(
     assert isinstance(response, controller.RegisterDebuggeeResponse)
 
 
-def test_register_debuggee_from_dict():
-    test_register_debuggee(request_type=dict)
-
-
 def test_register_debuggee_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -569,7 +707,9 @@ def test_register_debuggee_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee == data.Debuggee(id="id_value")
+        arg = args[0].debuggee
+        mock_val = data.Debuggee(id="id_value")
+        assert arg == mock_val
 
 
 def test_register_debuggee_flattened_error():
@@ -607,7 +747,9 @@ async def test_register_debuggee_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee == data.Debuggee(id="id_value")
+        arg = args[0].debuggee
+        mock_val = data.Debuggee(id="id_value")
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -622,9 +764,10 @@ async def test_register_debuggee_flattened_error_async():
         )
 
 
-def test_list_active_breakpoints(
-    transport: str = "grpc", request_type=controller.ListActiveBreakpointsRequest
-):
+@pytest.mark.parametrize(
+    "request_type", [controller.ListActiveBreakpointsRequest, dict,]
+)
+def test_list_active_breakpoints(request_type, transport: str = "grpc"):
     client = Controller2Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -654,10 +797,6 @@ def test_list_active_breakpoints(
     assert response.wait_expired is True
 
 
-def test_list_active_breakpoints_from_dict():
-    test_list_active_breakpoints(request_type=dict)
-
-
 def test_list_active_breakpoints_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -733,7 +872,9 @@ def test_list_active_breakpoints_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
 
 
 def test_list_active_breakpoints_flattened_error():
@@ -771,7 +912,9 @@ async def test_list_active_breakpoints_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -786,9 +929,10 @@ async def test_list_active_breakpoints_flattened_error_async():
         )
 
 
-def test_update_active_breakpoint(
-    transport: str = "grpc", request_type=controller.UpdateActiveBreakpointRequest
-):
+@pytest.mark.parametrize(
+    "request_type", [controller.UpdateActiveBreakpointRequest, dict,]
+)
+def test_update_active_breakpoint(request_type, transport: str = "grpc"):
     client = Controller2Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -814,10 +958,6 @@ def test_update_active_breakpoint(
     assert isinstance(response, controller.UpdateActiveBreakpointResponse)
 
 
-def test_update_active_breakpoint_from_dict():
-    test_update_active_breakpoint(request_type=dict)
-
-
 def test_update_active_breakpoint_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -891,8 +1031,12 @@ def test_update_active_breakpoint_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
-        assert args[0].breakpoint_ == data.Breakpoint(id="id_value")
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
+        arg = args[0].breakpoint_
+        mock_val = data.Breakpoint(id="id_value")
+        assert arg == mock_val
 
 
 def test_update_active_breakpoint_flattened_error():
@@ -932,8 +1076,12 @@ async def test_update_active_breakpoint_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
-        assert args[0].breakpoint_ == data.Breakpoint(id="id_value")
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
+        arg = args[0].breakpoint_
+        mock_val = data.Breakpoint(id="id_value")
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -970,6 +1118,23 @@ def test_credentials_transport_error():
             transport=transport,
         )
 
+    # It is an error to provide an api_key and a transport instance.
+    transport = transports.Controller2GrpcTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    options = client_options.ClientOptions()
+    options.api_key = "api_key"
+    with pytest.raises(ValueError):
+        client = Controller2Client(client_options=options, transport=transport,)
+
+    # It is an error to provide an api_key and a credential.
+    options = mock.Mock()
+    options.api_key = "api_key"
+    with pytest.raises(ValueError):
+        client = Controller2Client(
+            client_options=options, credentials=ga_credentials.AnonymousCredentials()
+        )
+
     # It is an error to provide scopes and a transport instance.
     transport = transports.Controller2GrpcTransport(
         credentials=ga_credentials.AnonymousCredentials(),
@@ -1437,7 +1602,7 @@ def test_parse_common_location_path():
     assert expected == actual
 
 
-def test_client_withDEFAULT_CLIENT_INFO():
+def test_client_with_default_client_info():
     client_info = gapic_v1.client_info.ClientInfo()
 
     with mock.patch.object(
@@ -1502,3 +1667,33 @@ def test_client_ctx():
             with client:
                 pass
             close.assert_called()
+
+
+@pytest.mark.parametrize(
+    "client_class,transport_class",
+    [
+        (Controller2Client, transports.Controller2GrpcTransport),
+        (Controller2AsyncClient, transports.Controller2GrpcAsyncIOTransport),
+    ],
+)
+def test_api_key_credentials(client_class, transport_class):
+    with mock.patch.object(
+        google.auth._default, "get_api_key_credentials", create=True
+    ) as get_api_key_credentials:
+        mock_cred = mock.Mock()
+        get_api_key_credentials.return_value = mock_cred
+        options = client_options.ClientOptions()
+        options.api_key = "api_key"
+        with mock.patch.object(transport_class, "__init__") as patched:
+            patched.return_value = None
+            client = client_class(client_options=options)
+            patched.assert_called_once_with(
+                credentials=mock_cred,
+                credentials_file=None,
+                host=client.DEFAULT_ENDPOINT,
+                scopes=None,
+                client_cert_source_for_mtls=None,
+                quota_project_id=None,
+                client_info=transports.base.DEFAULT_CLIENT_INFO,
+                always_use_jwt_access=True,
+            )
